#!/usr/bin/env python3
import argparse
import json
import os
from rouge import Rouge, FilesRouge

def main():
  import argparse
  parser = argparse.ArgumentParser(description='Rouge Metric Calculator')
  parser.add_argument('-f', '--file', help="File mode", action='store_true')
  parser.add_argument('-a', '--avg', help="Average mode", action='store_true')
  parser.add_argument('hypothesis', type=str, help='Text of file path')
  parser.add_argument('reference', type=str, help='Text or file path')

  args = parser.parse_args()
  if args.file:
    hyp, ref = args.hypothesis, args.reference
    assert(os.path.isfile(hyp))
    assert(os.path.isfile(ref))

    files_rouge = FilesRouge(hyp, ref)
    scores = files_rouge.get_scores(avg=args.avg)

    print(json.dumps(scores, indent=2))
  else:
    hyp, ref = args.hypothesis, args.reference
    assert(type(hyp) == str)
    assert(type(ref) == str)
    
    rouge = Rouge()
    scores = rouge.get_scores(hyp, ref, avg=args.avg)

    print(json.dumps(scores, indent=2))

if __name__ == "__main__":
  main()



# -*- coding: utf-8 -*-
import rouge.rouge_score as rouge_score
import os
import numpy as np

class FilesRouge:
  def __init__(self, hyp_path, ref_path, metrics=None, stats=None, batch_lines=None):
    assert(os.path.isfile(hyp_path))
    assert(os.path.isfile(ref_path))

    self.rouge = Rouge(metrics=metrics, stats=stats)

    def line_count(path):
      count = 0
      for line in open(path):
        count += 1
      return count

    hyp_lc = line_count(hyp_path)
    ref_lc = line_count(ref_path)
    assert(hyp_lc == ref_lc)

    assert(batch_lines is None or type(batch_lines) == int)

    self.hyp_path = hyp_path
    self.ref_path = ref_path
    self.batch_lines = batch_lines
  
  def get_scores(self, avg=False):
    """Calculate ROUGE scores between each pair of 
    lines (hyp_file[i], ref_file[i]).
    Args:
      * hyp_path: hypothesis file path
      * ref_path: references file path
      * avg (False): whether to get an average scores or a list
      * batch_line(None): set it to an integer value to work with
        subsets of `batch_line` lines (uses less memory)
    """
    batch_lines = self.batch_lines
    hyp_path, ref_path = self.hyp_path, self.ref_path

    if batch_lines is None:
      hyps = [line[:-1] for line in open(hyp_path).readlines()]
      refs = [line[:-1] for line in open(ref_path).readlines()]
     
      
      return self.rouge.get_scores(hyps, refs, avg=avg)

    else:
      if batch_lines > hyp_lc:
        batch_lines = hyp_lc
      
      if avg:
        sc = [0, 0, 0]
        update_scores = lambda s, h, r: [sum(x) for x in zip(s, self.rouge.get_scores(h, r, avg=True))]
      else:
        sc = []
        update_scores = lambda s, h, r: s + self.rouge.get_scores(batch_hyp, batch_ref)

      hyp_file = open(hyp_path)
      ref_file = open(ref_path)
      
      batch_hyp = []
      batch_ref = []

      for count in range(hyp_lc):
        batch_hyp.append(hyp_file.readline()[:-1])
        batch_ref.append(ref_file.readline()[:-1])

        count += 1
        if count == batch_lines:
          sc = update_scores(sc, batch_hyp, batch_ref)
          count = 0
          batch_hyp = []
          batch_ref = []

      if avg:
        return [s/hyp_lc for s in sc]
      return sc


class Rouge:
  DEFAULT_METRICS = ["rouge-1", "rouge-2", "rouge-l"]
  AVAILABLE_METRICS = {"rouge-1": lambda hyp, ref: rouge_score.rouge_n([hyp], [ref], 1), 
                       "rouge-2": lambda hyp, ref: rouge_score.rouge_n([hyp], [ref], 2),
                       "rouge-l": lambda hyp, ref: rouge_score.rouge_l_sentence_level([hyp], [ref]),
                      }

  DEFAULT_STATS = ["f", "p", "r"]
  AVAILABLE_STATS = {"f": 0, "p": 1, "r": 2
  }
  def __init__(self, metrics=None, stats=None):
    self.metrics = metrics if metrics is not None else Rouge.DEFAULT_METRICS
    self.stats = stats if stats is not None else Rouge.DEFAULT_STATS
    
    for m in self.metrics:
      if m not in Rouge.AVAILABLE_METRICS:
        raise ValueError("Unknown metric '%s'" % m)

    for s in self.stats:
      if s not in Rouge.AVAILABLE_STATS:
        raise ValueError("Unknown stat '%s'" % s)
  
  def get_scores(self, hyps, refs, avg=False):
    if type(hyps) == str:
      hyps, refs = [hyps], [refs]

    assert(type(hyps) == type(refs))
    assert(len(hyps) == len(refs))
    
    if not avg:
      return self._get_scores(hyps, refs)
    return self._get_avg_scores(hyps, refs)

  def _get_scores(self, hyps, refs):
    scores = []
    for hyp, ref in zip(hyps, refs):
      sen_score = {}
      for m in self.metrics:
        fn = Rouge.AVAILABLE_METRICS[m] 
        sc = fn(hyp, ref)
        sen_score[m] = {s: sc[Rouge.AVAILABLE_STATS[s]] for s in self.stats}
      scores.append(sen_score)
    return scores

  def _get_avg_scores(self, hyps, refs):
    scores = {}
    for m in self.metrics:
      fn = Rouge.AVAILABLE_METRICS[m]
      sc = [fn(hyp, ref) for hyp, ref in zip(hyps, refs)]
      sc = [[sen_sc[Rouge.AVAILABLE_STATS[s]] for s in self.stats] for sen_sc in sc]
      scores[m] = {s: st for s, st in zip(self.stats, tuple(map(np.mean, zip(*sc))))}
    return scores





from unittest import TestCase

import rouge
import json

class BasicTest(TestCase):
  def setUp(self):
    self.hyp_path = './tests/hyp.txt'
    self.ref_path = './tests/ref.txt'

    self.data_path = './tests/data.json' 
    with open(self.data_path) as f:
      self.data = json.load(f)
    
    self.rouge = rouge.Rouge()
    self.files_rouge = rouge.FilesRouge(self.hyp_path, self.ref_path)

  def test_one_sentence(self):
    for d in self.data[:1]:
      hyp = d["hyp"]
      ref = d["ref"]
      score = self.rouge.get_scores(hyp, ref)[0]
      self.assertEqual(score, d["scores"])
  
  def test_multi_sentence(self):
    data = self.data
    hyps, refs = map(list, zip(*[[d['hyp'], d['ref']] for d in data]))
    expected_scores = [d['scores'] for d in data]
    scores = self.rouge.get_scores(hyps, refs)
    self.assertEqual(expected_scores, scores)

  def test_files_scores(self):
    data = self.data
    hyps, refs = map(list, zip(*[[d['hyp'], d['ref']] for d in data]))
    expected_scores = [d['scores'] for d in data]
    scores = self.files_rouge.get_scores()
    self.assertEqual(expected_scores, scores)



from setuptools import setup, find_packages
import rouge

version = rouge.__version__
setup(
    name="rouge",
    version=version,
    description="Full Python ROUGE Score Implementation (not a wrapper)",
    url="http://github.com/pltrdy/rouge",
    download_url="https://github.com/pltrdy/rouge/archive/%s.tar.gz" % version,
    author="pltrdy",
    author_email="pltrdy@gmail.com",
    keywords=["NL", "CL", "natural language processing", "computational linguistics", "summarization"],
    packages=find_packages(),
    classifiers=[
        "Intended Audience :: Science/Research",
        "Programming Language :: Python :: 3",
        "Topic :: Text Processing :: Linguistic"
        ],
    license="LICENCE.txt",
    long_description=open("README.md").read(),
    test_suite="nose.collector",
    tests_require=['nose'],

    entry_points={
      'console_scripts': [
        'rouge=bin.rouge_cmd:main'
      ]
    }
)
